﻿ # Copyright 2016 Hewlett Packard Enterprise Development, LP.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
 #
 #      http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.


import sys
import json
import logging
import os
import pprint
import redfish.ris.tpdefs

from redfish import AuthMethod, redfish_client
#from redfish import redfish_logger

#LOGGERFILE = "esmsmsrvpy.log"
#LOGGERFORMAT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
#LOGGER = redfish_logger(LOGGERFILE, LOGGERFORMAT, logging.INFO)
#LOGGER.info("esmsmsrv.py log")

# Global Session
Redfish_OBJ = None

def get_resource_directory(redfishobj):
    response = redfishobj.redfish_get("/redfish/v1/resourcedirectory/")
    resources = {}

    #If 200 is returning, it succeeds.
    if response.status == 200:
        resources["resources"] = response.dict["Instances"]
    else:
        resources["resources"] = {}

    return resources

def dump_iml(redfishobj):
    lt = list()

    #search of LogService ResourceURI
    instances = redfishobj.search_for_type("LogService.")

    #404 is abnormal:The construction of REST changed.
    if not instances:
        return 404, lt

    for instance in instances:
        if instance[redfishobj.typepath.defs.hrefstring].endswith("IML/"):
            res_iml = redfishobj.redfish_get(instance[redfishobj.typepath.defs.hrefstring])
            if res_iml.status != 200:
                return res_iml.status,lt

            uristr = res_iml.dict["Entries"][redfishobj.typepath.defs.hrefstring]
            entries_res = redfishobj.redfish_get(uristr)
            if entries_res.status != 200:
               return entries_res.status,lt

            for iml_entry in entries_res.dict[redfishobj.typepath.defs.collectionstring]:
               entry_res = redfishobj.redfish_get(iml_entry[redfishobj.typepath.defs.hrefstring])
               if entry_res.status != 200:
                   return entry_res.status, lt

               dt = dict()

               #IML information is stocked in a return list.
               if ("Oem" in entry_res.dict):
                   for oem in entry_res.dict["Oem"].values():
                       if "EventNumber" in oem:
                           dt["EventNumber"] = oem["EventNumber"]
                       if "Class" in oem:
                           dt["Class"] = oem["Class"]
                       if "Code" in oem:
                           dt["Code"] = oem["Code"]
                       if "Updated" in oem:
                           dt["Updated"] = oem["Updated"]
                       if "Repaired" in oem:
                           dt["Repaired"] = oem["Repaired"]
                       if "Categories" in oem:
                           dt["Categories"] = oem["Categories"]
                       if "RecommendedAction" in oem:
                           dt["RecommendedAction"] = oem["RecommendedAction"]
                    
               if "Severity" in entry_res.dict:
                    dt["Severity"] =entry_res.dict["Severity"]

               if "Message" in entry_res.dict:
                   dt["Message"] = entry_res.dict["Message"]

               if "Created" in entry_res.dict:
                   dt["Created"] = entry_res.dict["Created"]

               lt.append(dt)

    return 200, lt


def get_iml():

    # When running remotely connect using the address, account name, 
    # and password to send https requests
    iLO_host = "blobstore://."
    login_account = None
    login_password = None

     #check expired
    __checksession()

    # Create a REST object
    global Redfish_OBJ
    if Redfish_OBJ is None:
    	Redfish_OBJ = RedfishObject(iLO_host, login_account, login_password)

    try: 
        status, imls = dump_iml(Redfish_OBJ)
    except:
        __delsession()
        raise

    if status != 200 :
        __delsession()

    return status, imls

def get_datetime():

    # When running remotely connect using the address, account name, 
    # and password to send https requests
    iLO_host = "blobstore://."
    login_account = None
    login_password = None

    #check expired
    __checksession()

    # Create a REST object
    global Redfish_OBJ
    if Redfish_OBJ is None:
    	Redfish_OBJ = RedfishObject(iLO_host, login_account, login_password)

    try:
        res_date = Redfish_OBJ.redfish_get("/redfish/v1/Managers/1/DateTime/")
    except: 
        __delsession()
        raise

    if res_date.status != 200 :
        __delsession()

    if "DateTime" in res_date.dict:
        return res_date.status, res_date.dict["DateTime"]

    return res_date.status, ""


def __delsession():
    global Redfish_OBJ
    if Redfish_OBJ is None:
        return

    del Redfish_OBJ
    Redfish_OBJ = None

    return

def __checksession():
    global Redfish_OBJ
    if Redfish_OBJ is None:
        return

    str = Redfish_OBJ.redfish_client.get_session_location()
    if str is None:
        __delsession()
        return

    start = str.find("/redfish/v1/")
    if start == -1:
        __delsession()
        return

    url = str[start : ]

    try:
        res_date = Redfish_OBJ.redfish_get(url)
    except: 
        __delsession()
        raise

    if res_date.status == 404:
        __delsession()

    return

def main():
    # When running remotely connect using the address, account name, 
    # and password to send https requests
    iLO_host = "blobstore://."
    login_account = None
    login_password = None

    # Create a Redfish object
    Redfish_OBJ = RedfishObject(iLO_host, login_account, login_password)
    st,lt = dump_iml(Redfish_OBJ)
    
    #IML information is output.
    sys.stdout.write("Status:" + str(st) + "\n")
    pprint.pprint(lt)

class RedfishObject(object):
    def __init__(self, host, login_account, login_password):
        self.redfish_client = redfish_client(base_url=host, \
                          username=login_account, password=login_password, \
                          default_prefix="/redfish/v1")
        self.typepath = redfish.ris.tpdefs.Typesandpathdefines()
        self.typepath.getgen(url=host, logger=None)
        self.typepath.defs.redfishchange()
        self.redfish_client.login(auth=AuthMethod.SESSION)
        self.SYSTEMS_RESOURCES = get_resource_directory(self)

    def __del__(self):
        try:
            self.redfish_client.logout()
        except AttributeError, excp:
            pass

    def search_for_type(self, type):
        instances = []

        for item in self.SYSTEMS_RESOURCES["resources"]:
            foundsettings = False

            if self.typepath.defs.typestring in item and type.lower() in \
                                                    item[self.typepath.defs.typestring].lower():
                for entry in self.SYSTEMS_RESOURCES["resources"]:
                    if (item[self.typepath.defs.hrefstring] + "/settings/").lower() == \
                                                (entry[self.typepath.defs.hrefstring]).lower():
                        foundsettings = True

                if not foundsettings:
                    instances.append(item)

        return instances

    def redfish_get(self, suburi):
        """REDFISH GET"""
        return self.redfish_client.get(path=suburi)

if __name__ == "__main__":
    main()
    