#!/bin/sh

#################################################################
#			Ver1.0 2016.08.31			#
#################################################################

SA_DIR="/opt/nec/esmpro_sa"
LOGPATH="${SA_DIR}/registry/HKEY_LOCAL_MACHINE/SOFTWARE/NEC/ESMAGENT/LOG"
TARGET=""
LOGLEVEL=0
FILEPATH=""
MAXLOGSIZE=0
GENERATION=0
TARGET_FLAG=0
LOGLEVEL_FLAG=0
FILEPATH_FLAG=0
MAXLOGSIZE_FLAG=0
GENERATION_FLAG=0

usage()
{
	/bin/echo "USAGE: `/bin/basename ${0}` -t TARGET [-l LEVEL] [-p PATH] [-m MAXSIZE] [-g GENERATION] [-h]"
	/bin/echo "	-t: change target(required!), must input esmsmsrv or esmsmsrv_internal."
	/bin/echo "	-l: log level in the range of 0 to 5."
	/bin/echo "	-p: log file's full path within 503 bytes."
	/bin/echo "	-m: MAX log size(KB) in the range of 1 to 102400."
	/bin/echo "	-g: generation of log file in the range of 0 to 10."
	/bin/echo "	-h: display the help information."
	/bin/echo "e.g.: ./esmchangelog.sh -t esmsmsrv -l 3 -p \"/tmp/test.log\" -m 512 -g 2"
	/bin/echo "      ./esmchangelog.sh -t esmsmsrv_internal -l 5 -p \"/tmp/test.log\" -m 1024 -g 0"
}


changeLogLevel()
{
	if [ -z "${LOGLEVEL##*[!0-9]*}" ] || [ "${LOGLEVEL}" -lt 0 ] || [ "${LOGLEVEL}" -gt 5 ]; then
		/bin/echo "Change log level failed, please input log level in the range of 0 to 5."
	else
		REGLOGLEVEL="${TARGETPATH}/V_LOGLEVEL"
		/bin/echo -ne "\x04\x00\x00\x00\x04\x00\x00\x00" > "${REGLOGLEVEL}" 2>/dev/null
		/bin/echo -ne "\x${LOGLEVEL}\x00\x00\x00" >> "${REGLOGLEVEL}" 2>/dev/null
		/bin/echo "Change log level success."
	fi

}

changeFilePath()
{
	LEN=${#FILEPATH}
	if [ "${LEN}" -lt 1 ] || [ "${LEN}" -gt 503 ]; then
		/bin/echo "Change file path failed, please input log file's full path within 503 bytes."
	else
		REGFILEPATH="${TARGETPATH}/V_FILEPATH"
		x=$[LEN/256]
		x=`printf %x ${x}`
		y=$[LEN%256]
		y=`printf %x ${y}`
		/bin/echo -ne "\x01\x00\x00\x00\x${y}\x${x}\x00\x00" > "${REGFILEPATH}" 2>/dev/null
		/bin/echo -ne "${FILEPATH}" >> "${REGFILEPATH}" 2>/dev/null
		/bin/echo "Change file path success."
	fi

}

changeMaxLogSize()
{
	if [ -z "${MAXLOGSIZE##*[!0-9]*}" ] || [ "${MAXLOGSIZE}" -lt 1 ] || [ "${MAXLOGSIZE}" -gt 102400 ]; then
		/bin/echo "Change max file size failed, please input max log size in the range of 1 to 102400(KB)."
	else
		REGMAXLOGSIZE="${TARGETPATH}/V_MAXLOGSIZE"
		MAXLOGSIZE=$[MAXLOGSIZE*1024]
		x=$[MAXLOGSIZE/16777216]
		x=`printf %x ${x}`
		MAXLOGSIZE=$[MAXLOGSIZE%16777216]
		y=$[MAXLOGSIZE/65536]
		y=`printf %x ${y}`
		MAXLOGSIZE=$[MAXLOGSIZE%65536]
		z=$[MAXLOGSIZE/256]
		z=`printf %x ${z}`
		/bin/echo -ne "\x04\x00\x00\x00\x04\x00\x00\x00" > "${REGMAXLOGSIZE}" 2>/dev/null
		/bin/echo -ne "\x00\x${z}\x${y}\x${x}" >> "${REGMAXLOGSIZE}" 2>/dev/null
		/bin/echo "Change max file size success."
	fi

}

changeGeneration()
{
	if [ -z "${GENERATION##*[!0-9]*}" ] || [ "${GENERATION}" -lt 0 ] || [ "${GENERATION}" -gt 10 ]; then
		/bin/echo "Change generation failed, please input generation of log file in the range of 0 to 10."
	else
		REGGENERATION="${TARGETPATH}/V_GENERATION"
		GENERATION=`printf %x ${GENERATION}`
		/bin/echo -ne "\x04\x00\x00\x00\x04\x00\x00\x00" > "${REGGENERATION}" 2>/dev/null
		/bin/echo -ne "\x${GENERATION}\x00\x00\x00" >> "${REGGENERATION}" 2>/dev/null
		/bin/echo "Change generation success."
	fi

}

changeLogMain()
{
	if [ "${TARGET_FLAG}" -eq 0 ] ; then
		usage
		/bin/echo "Please input the target with esmsmsrv or esmsmsrv_internal."
		exit 1
	fi

	case ${TARGET} in
		"esmsmsrv")            TARGETPATH="${LOGPATH}/ESMSMSRV"
			               ;;
		"esmsmsrv_internal")   TARGETPATH="${LOGPATH}/ESMSMSRV_INTERNAL"
			               ;;
		*)	               usage
				       /bin/echo "Please input the target with esmsmsrv or esmsmsrv_internal."
			               exit 1
			               ;;
	esac

        if [ ! -d "${TARGETPATH}" ] ; then
                echo "ERROR: ${TARGETPATH} is not exist."
		exit 1
        fi

	if [ "${LOGLEVEL_FLAG}" -eq 1 ] ; then
		changeLogLevel
	fi

	if [ "${FILEPATH_FLAG}" -eq 1 ] ; then
		changeFilePath
	fi

	if [ "${MAXLOGSIZE_FLAG}" -eq 1 ] ; then
		changeMaxLogSize
	fi

	if [ "${GENERATION_FLAG}" -eq 1 ] ; then
		changeGeneration
	fi

	echo "End."
}

#process
while [ "$1" != "" ] ; do
	case "$1" in
		"-t")
			TARGET="$2"
			TARGET_FLAG=1
			shift
			;;
		"-l")
			LOGLEVEL="$2"
			LOGLEVEL_FLAG=1
			shift
			;;
		"-p")
			FILEPATH="$2"
			FILEPATH_FLAG=1
			shift
			;;
		"-m")
			MAXLOGSIZE="$2"
			MAXLOGSIZE_FLAG=1
			shift
			;;
		"-g")
			GENERATION="$2"
			GENERATION_FLAG=1
			shift
			;;
		"-h")
			usage
			exit 0
			;;
		*)
			usage
			exit 1
			;;
	esac
	shift
done

changeLogMain
